@@ -1,5 +1,5 @@
 #
-# Copyright 2005-2016 ECMWF.
+# Copyright 2005-2017 ECMWF.
 #
 # This software is licensed under the terms of the Apache Licence Version 2.0
 # which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -19,7 +19,7 @@ VERBOSE = 1  # verbose error reporting
 
 def example():
     f = open(sys.argv[1])
-    out = open(sys.argv[2], 'w')
+    fout = open(sys.argv[2], 'w')
 
     while 1:
         gid = codes_grib_new_from_file(f)
@@ -29,12 +29,12 @@ def example():
         message = codes_get_message(gid)
 
         newgid = codes_new_from_message(message)
-        codes_write(newgid, out)
+        codes_write(newgid, fout)
         codes_release(newgid)
 
         codes_release(gid)
 
-    out.close()
+    fout.close()
     f.close()
 
 
