@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2016 ECMWF.
+ * Copyright 2005-2017 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -21,26 +21,30 @@ typedef struct bits_all_one_t {
 
 static bits_all_one_t bits_all_one={0,0,{0,}};
 
-static void init_bits_all_one() {
+static void init_bits_all_one()
+{
     int size=sizeof(long)*8;
     long* v=0;
     unsigned long cmask=-1;
     bits_all_one.size=size;
     bits_all_one.inited=1;
     v=bits_all_one.v+size;
-    *v= cmask << size;
+    /*
+     * The result of a shift operation is undefined if the RHS is negative or
+     * greater than or equal to the number of bits in the (promoted) shift-expression
+     */
+    /* *v= cmask << size; */
+    *v = -1;
     while (size>0)  *(--v)= ~(cmask << --size);
-
 }
 
-int grib_is_all_bits_one(long val, long nbits) {
-
+int grib_is_all_bits_one(long val, long nbits)
+{
     if (!bits_all_one.inited) init_bits_all_one();
     return bits_all_one.v[nbits]==val;
-
 }
 
-int grib_encode_string(const unsigned char* bitStream, long *bitOffset, size_t numberOfCharacters,char* string)
+int grib_encode_string(unsigned char *bitStream, long *bitOffset, size_t numberOfCharacters, const char *string)
 {
     size_t i;
     int err=0;
@@ -114,7 +118,7 @@ char* grib_decode_string(const unsigned char* bitStream, long *bitOffset, size_t
 /* A mask with x least-significant bits set, possibly 0 or >=32 */
 /* -1UL is 1111111... in every bit in binary representation */
 #define BIT_MASK(x) \
-        (((x) >= sizeof(unsigned long) * 8) ? \
+        (((x) >= max_nbits) ? \
                 (unsigned long) -1UL : (1UL << (x)) - 1)
 /**
  * decode a value consisting of nbits from an octet-bitstream to long-representation
@@ -278,16 +282,9 @@ int grib_encode_unsigned_longb(unsigned char* p, unsigned long val ,long *bitp,
 }
 
 #if OMP_PACKING
-
-#include "grib_bits_any_endian_omp.c"
-
+ #include "grib_bits_any_endian_omp.c"
 #elif VECTOR
-
-#include "grib_bits_any_endian_vector.c"
-
+ #include "grib_bits_any_endian_vector.c"
 #else
-
-#include "grib_bits_any_endian_simple.c"
-
+ #include "grib_bits_any_endian_simple.c"
 #endif
-
