@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2016 ECMWF.
+ * Copyright 2005-2017 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -95,6 +95,8 @@ grib_dumper_class* grib_dumper_class_bufr_decode_fortran = &_grib_dumper_class_b
 /* END_CLASS_IMP */
 static void dump_attributes(grib_dumper* d, grib_accessor* a, const char* prefix);
 
+/* Note: A fast cut-down version of strcmp which does NOT return -1 */
+/* 0 means input strings are equal and 1 means not equal */
 GRIB_INLINE static int grib_inline_strcmp(const char* a, const char* b)
 {
     if (*a != *b) return 1;
@@ -122,7 +124,7 @@ static int init(grib_dumper* d)
     d->count=1;
     self->isLeaf=0;
     self->isAttribute=0;
-    self->keys=grib_context_malloc_clear(c,sizeof(grib_string_list));
+    self->keys=(grib_string_list*)grib_context_malloc_clear(c,sizeof(grib_string_list));
 
     return GRIB_SUCCESS;
 }
@@ -167,12 +169,12 @@ static void dump_values(grib_dumper* d, grib_accessor* a)
     if (size>1) {
         depth-=2;
 
-        if ((r=compute_key_rank(h,self->keys,a->name))!=0)
+        if ((r=compute_bufr_key_rank(h,self->keys,a->name))!=0)
             fprintf(self->dumper.out,"  call codes_get(ibufr, '#%d#%s', rValues)\n",r,a->name);
         else
             fprintf(self->dumper.out,"  call codes_get(ibufr, '%s', rValues)\n",a->name);
     } else {
-        r=compute_key_rank(h,self->keys,a->name);
+        r=compute_bufr_key_rank(h,self->keys,a->name);
         if( !grib_is_missing_double(a,value) ) {
             if (r!=0)
                 fprintf(self->dumper.out,"  call codes_get(ibufr, '#%d#%s', rVal)\n", r, a->name);
@@ -186,7 +188,7 @@ static void dump_values(grib_dumper* d, grib_accessor* a)
         int dofree=0;
 
         if (r!=0) {
-            prefix=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
+            prefix=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
             dofree=1;
             sprintf(prefix,"#%d#%s",r,a->name);
         } else prefix=(char*)a->name;
@@ -230,7 +232,7 @@ static void dump_values_attribute(grib_dumper* d, grib_accessor* a, const char*
     if (self->isLeaf==0) {
         char* prefix1;
 
-        prefix1=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+strlen(prefix)+5));
+        prefix1=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+strlen(prefix)+5));
         sprintf(prefix1,"%s->%s",prefix,a->name);
 
         dump_attributes(d,a,prefix1);
@@ -262,9 +264,9 @@ static void dump_long(grib_dumper* d,grib_accessor* a, const char* comment)
             char* prefix;
             int dofree=0;
 
-            r=compute_key_rank(h,self->keys,a->name);
+            r=compute_bufr_key_rank(h,self->keys,a->name);
             if (r!=0) {
-                prefix=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
+                prefix=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
                 dofree=1;
                 sprintf(prefix,"#%d#%s",r,a->name);
             } else prefix=(char*)a->name;
@@ -286,13 +288,13 @@ static void dump_long(grib_dumper* d,grib_accessor* a, const char* comment)
         depth-=2;
         fprintf(self->dumper.out,"  if(allocated(iValues)) deallocate(iValues)\n");
 
-        if ((r=compute_key_rank(h,self->keys,a->name))!=0)
+        if ((r=compute_bufr_key_rank(h,self->keys,a->name))!=0)
             fprintf(self->dumper.out,"  call codes_get(ibufr, '#%d#%s', iValues)\n", r,a->name);
         else
             fprintf(self->dumper.out,"  call codes_get(ibufr, '%s', iValues)\n", a->name);
 
     } else {
-        r=compute_key_rank(h,self->keys,a->name);
+        r=compute_bufr_key_rank(h,self->keys,a->name);
         if( !grib_is_missing_long(a,value) ) {
             if (r!=0)
                 fprintf(self->dumper.out,"  call codes_get(ibufr, '#%d#%s', iVal)\n", r, a->name);
@@ -306,7 +308,7 @@ static void dump_long(grib_dumper* d,grib_accessor* a, const char* comment)
         int dofree=0;
 
         if (r!=0) {
-            prefix=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
+            prefix=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
             dofree=1;
             sprintf(prefix,"#%d#%s",r,a->name);
         } else prefix=(char*)a->name;
@@ -353,7 +355,7 @@ static void dump_long_attribute(grib_dumper* d, grib_accessor* a, const char* pr
     if (self->isLeaf==0) {
         char* prefix1;
 
-        prefix1=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+strlen(prefix)+5));
+        prefix1=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+strlen(prefix)+5));
         sprintf(prefix1,"%s->%s",prefix,a->name);
 
         dump_attributes(d,a,prefix1);
@@ -382,7 +384,7 @@ static void dump_double(grib_dumper* d, grib_accessor* a, const char* comment)
 
     self->empty=0;
 
-    r=compute_key_rank(h,self->keys,a->name);
+    r=compute_bufr_key_rank(h,self->keys,a->name);
     if( !grib_is_missing_double(a,value) ) {
         if (r!=0)
             fprintf(self->dumper.out,"  call codes_get(ibufr,'#%d#%s', rVal)\n",r,a->name);
@@ -395,7 +397,7 @@ static void dump_double(grib_dumper* d, grib_accessor* a, const char* comment)
         int dofree=0;
 
         if (r!=0) {
-            prefix=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
+            prefix=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
             dofree=1;
             sprintf(prefix,"#%d#%s",r,a->name);
         } else prefix=(char*)a->name;
@@ -434,7 +436,7 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
     self->empty=0;
 
     if (self->isLeaf==0) {
-        if ((r=compute_key_rank(h,self->keys,a->name))!=0)
+        if ((r=compute_bufr_key_rank(h,self->keys,a->name))!=0)
             fprintf(self->dumper.out,"  call codes_get_string_array(ibufr,'#%d#%s',sValues)\n",r,a->name);
         else
             fprintf(self->dumper.out,"  call codes_get_string_array(ibufr,'%s',sValues)\n",a->name);
@@ -445,7 +447,7 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
         int dofree=0;
 
         if (r!=0) {
-            prefix=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
+            prefix=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
             dofree=1;
             sprintf(prefix,"#%d#%s",r,a->name);
         } else prefix=(char*)a->name;
@@ -485,7 +487,7 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
 
     err = grib_unpack_string(a,value,&size);
     p=value;
-    r=compute_key_rank(h,self->keys,a->name);
+    r=compute_bufr_key_rank(h,self->keys,a->name);
     if (grib_is_missing_string(a,(unsigned char *)value,size))
         return;
 
@@ -506,7 +508,7 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
         int dofree=0;
 
         if (r!=0) {
-            prefix=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
+            prefix=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
             dofree=1;
             sprintf(prefix,"#%d#%s",r,a->name);
         } else prefix=(char*)a->name;
