@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2016 ECMWF.
+ * Copyright 2005-2017 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -94,6 +94,8 @@ grib_dumper_class* grib_dumper_class_bufr_encode_filter = &_grib_dumper_class_bu
 /* END_CLASS_IMP */
 static void dump_attributes(grib_dumper* d,grib_accessor* a, const char* prefix);
 
+/* Note: A fast cut-down version of strcmp which does NOT return -1 */
+/* 0 means input strings are equal and 1 means not equal */
 GRIB_INLINE static int grib_inline_strcmp(const char* a, const char* b)
 {
     if (*a != *b) return 1;
@@ -120,7 +122,7 @@ static int init(grib_dumper* d)
     self->empty=1;
     self->isLeaf=0;
     self->isAttribute=0;
-    self->keys=grib_context_malloc_clear(c,sizeof(grib_string_list));
+    self->keys=(grib_string_list*)grib_context_malloc_clear(c,sizeof(grib_string_list));
 
     return GRIB_SUCCESS;
 }
@@ -171,7 +173,7 @@ static void dump_values(grib_dumper* d, grib_accessor* a)
     if (size>1) {
         int icount=0;
 
-        if ((r=compute_key_rank(h,self->keys,a->name))!=0)
+        if ((r=compute_bufr_key_rank(h,self->keys,a->name))!=0)
             fprintf(self->dumper.out,"set #%d#%s=",r,a->name);
         else
             fprintf(self->dumper.out,"set %s=",a->name);
@@ -190,7 +192,7 @@ static void dump_values(grib_dumper* d, grib_accessor* a)
         fprintf(self->dumper.out,"};\n");
         grib_context_free(c,values);
     } else {
-        r=compute_key_rank(h,self->keys,a->name);
+        r=compute_bufr_key_rank(h,self->keys,a->name);
         if( !grib_is_missing_double(a,value) ) {
 
             if (r!=0)
@@ -207,7 +209,7 @@ static void dump_values(grib_dumper* d, grib_accessor* a)
         int dofree=0;
 
         if (r!=0) {
-            prefix=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
+            prefix=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
             dofree=1;
             sprintf(prefix,"#%d#%s",r,a->name);
         } else prefix=(char*)a->name;
@@ -261,7 +263,7 @@ static void dump_values_attribute(grib_dumper* d,grib_accessor* a, const char* p
         fprintf(self->dumper.out,"};\n");
         grib_context_free(c,values);
     } else {
-        /* int r=compute_key_rank(h,self->keys,a->name); */
+        /* int r=compute_bufr_key_rank(h,self->keys,a->name); */
         if( !grib_is_missing_double(a,value) ) {
             fprintf(self->dumper.out,"set %s->%s = %.18e;\n", prefix, a->name, value);
         }
@@ -270,7 +272,7 @@ static void dump_values_attribute(grib_dumper* d,grib_accessor* a, const char* p
     if (self->isLeaf==0) {
         char* prefix1;
 
-        prefix1=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+strlen(prefix)+5));
+        prefix1=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+strlen(prefix)+5));
         sprintf(prefix1,"%s->%s",prefix,a->name);
 
         dump_attributes(d,a,prefix1);
@@ -304,9 +306,9 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
             char* prefix;
             int dofree=0;
             
-            r=compute_key_rank(h,self->keys,a->name);
+            r=compute_bufr_key_rank(h,self->keys,a->name);
             if (r!=0) {
-                prefix=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
+                prefix=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
                 dofree=1;
                 sprintf(prefix,"#%d#%s",r,a->name);
             } else prefix=(char*)a->name;
@@ -330,7 +332,7 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
 
     if (size>1) {
         icount=0;
-        if ((r=compute_key_rank(h,self->keys,a->name))!=0)
+        if ((r=compute_bufr_key_rank(h,self->keys,a->name))!=0)
             fprintf(self->dumper.out,"set #%d#%s=",r,a->name);
         else
             fprintf(self->dumper.out,"set %s=",a->name);
@@ -349,7 +351,7 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
         fprintf(self->dumper.out,"};\n");
         grib_context_free(a->context,values);
     } else {
-        r=compute_key_rank(h,self->keys,a->name);
+        r=compute_bufr_key_rank(h,self->keys,a->name);
         if( !grib_is_missing_long(a,value) ) {
             if (r!=0)
                 fprintf(self->dumper.out,"set #%d#%s=",r,a->name);
@@ -365,7 +367,7 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
         int dofree=0;
 
         if (r!=0) {
-            prefix=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
+            prefix=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
             dofree=1;
             sprintf(prefix,"#%d#%s",r,a->name);
         } else prefix=(char*)a->name;
@@ -418,7 +420,7 @@ static void dump_long_attribute(grib_dumper* d, grib_accessor* a, const char* pr
         grib_context_free(a->context,values);
 
     } else {
-        /* int r=compute_key_rank(h,self->keys,a->name); */
+        /* int r=compute_bufr_key_rank(h,self->keys,a->name); */
         if( !grib_is_missing_long(a,value) ) {
             fprintf(self->dumper.out,"set %s->%s = ",prefix,a->name);
             fprintf(self->dumper.out,"%ld ;\n",value);
@@ -428,7 +430,7 @@ static void dump_long_attribute(grib_dumper* d, grib_accessor* a, const char* pr
     if (self->isLeaf==0) {
         char* prefix1;
 
-        prefix1=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+strlen(prefix)+5));
+        prefix1=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+strlen(prefix)+5));
         sprintf(prefix1,"%s->%s",prefix,a->name);
 
         dump_attributes(d,a,prefix1);
@@ -458,7 +460,7 @@ static void dump_double(grib_dumper* d, grib_accessor* a, const char* comment)
     self->begin=0;
     self->empty=0;
 
-    r=compute_key_rank(h,self->keys,a->name);
+    r=compute_bufr_key_rank(h,self->keys,a->name);
     if( !grib_is_missing_double(a,value) ) {
         if (r!=0)
             fprintf(self->dumper.out,"set #%d#%s=",r,a->name);
@@ -473,7 +475,7 @@ static void dump_double(grib_dumper* d, grib_accessor* a, const char* comment)
         int dofree=0;
 
         if (r!=0) {
-            prefix=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
+            prefix=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
             dofree=1;
             sprintf(prefix,"#%d#%s",r,a->name);
         } else prefix=(char*)a->name;
@@ -511,7 +513,7 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
 
     if (self->isLeaf==0) {
         depth+=2;
-        if ((r=compute_key_rank(h,self->keys,a->name))!=0)
+        if ((r=compute_bufr_key_rank(h,self->keys,a->name))!=0)
             fprintf(self->dumper.out,"set #%d#%s=",r,a->name);
         else
             fprintf(self->dumper.out,"set %s=",a->name);
@@ -543,7 +545,7 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
         int dofree=0;
 
         if (r!=0) {
-            prefix=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
+            prefix=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
             dofree=1;
             sprintf(prefix,"#%d#%s",r,a->name);
         } else prefix=(char*)a->name;
@@ -586,7 +588,7 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
 
     err = grib_unpack_string(a,value,&size);
     p=value;
-    r=compute_key_rank(h,self->keys,a->name);
+    r=compute_bufr_key_rank(h,self->keys,a->name);
     if (grib_is_missing_string(a,(unsigned char *)value,size))
         return;
 
@@ -607,7 +609,7 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
         int dofree=0;
 
         if (r!=0) {
-            prefix=grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
+            prefix=(char*)grib_context_malloc_clear(c,sizeof(char)*(strlen(a->name)+10));
             dofree=1;
             sprintf(prefix,"#%d#%s",r,a->name);
         } else prefix=(char*)a->name;
@@ -637,7 +639,7 @@ static void _dump_long_array(grib_handle* h, FILE* f, const char* key, const cha
 
     if (grib_get_size(h,key,&size)==GRIB_NOT_FOUND) return;
 
-    val=grib_context_malloc_clear(h->context,sizeof(long)*size);
+    val=(long*)grib_context_malloc_clear(h->context,sizeof(long)*size);
     grib_get_long_array(h,key,val,&size);
     fprintf(f,"set %s= {",print_key);
     for (i=0;i<size-1;i++) {
