@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2016 ECMWF.
+ * Copyright 2005-2017 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -92,6 +92,8 @@ grib_dumper_class* grib_dumper_class_json = &_grib_dumper_class_json;
 /* END_CLASS_IMP */
 static void dump_attributes(grib_dumper* d,grib_accessor* a);
 
+/* Note: A fast cut-down version of strcmp which does NOT return -1 */
+/* 0 means input strings are equal and 1 means not equal */
 GRIB_INLINE static int grib_inline_strcmp(const char* a,const char* b)
 {
     if (*a != *b) return 1;
@@ -327,7 +329,7 @@ static void dump_double(grib_dumper* d,grib_accessor* a,const char* comment)
 static void dump_string_array(grib_dumper* d,grib_accessor* a,const char* comment)
 {
     grib_dumper_json *self = (grib_dumper_json*)d;
-    char **values;
+    char **values = NULL;
     size_t size = 0,i=0;
     grib_context* c=NULL;
     int err = 0;
@@ -371,7 +373,7 @@ static void dump_string_array(grib_dumper* d,grib_accessor* a,const char* commen
     }
     fprintf(self->dumper.out,"\n%-*s[",depth," ");
     depth+=2;
-    for  (i=0;i<size-1;i++) {
+    for (i=0;i<size-1;i++) {
         fprintf(self->dumper.out,"%-*s\"%s\",\n",depth," ",values[i]);
     }
     fprintf(self->dumper.out,"%-*s\"%s\"\n",depth," ",values[i]);
@@ -387,6 +389,9 @@ static void dump_string_array(grib_dumper* d,grib_accessor* a,const char* commen
         fprintf(self->dumper.out,"\n%-*s}",depth," ");
     }
 
+    for (i=0;i<size;i++) {
+        grib_context_free(c, values[i]);
+    }
     grib_context_free(c,values);
     (void)err; /* TODO */
 }
