@@ -1,5 +1,5 @@
 #!/bin/sh
-# Copyright 2005-2016 ECMWF.
+# Copyright 2005-2017 ECMWF.
 #
 # This software is licensed under the terms of the Apache Licence Version 2.0
 # which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -23,11 +23,11 @@ do
   fi
 
   ${test_dir}/multi_from_message -m $infile > $out1
-  ${tools_dir}grib_get -p count,shortName,level $infile > $out2
+  ${tools_dir}/grib_get -p count,shortName,level $infile > $out2
   diff $out1 $out2
 
   ${test_dir}/multi_from_message $infile > $out1
-  ${tools_dir}grib_get -M -p count,shortName,level $infile > $out2
+  ${tools_dir}/grib_get -M -p count,shortName,level $infile > $out2
   diff $out1 $out2
 
   rm -f $out1 $out2 | true
