@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2016 ECMWF.
+ * Copyright 2005-2017 ECMWF.
  *
  * This software is licensed under the terms of the Apache Licence Version 2.0
  * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
@@ -21,7 +21,6 @@
 #undef PACKAGE_STRING
 #undef PACKAGE_TARNAME
 #undef PACKAGE_VERSION
-#include "jasper/jasper.h"
 #endif
 
 #ifdef ENABLE_FLOATING_POINT_EXCEPTIONS
@@ -117,24 +116,24 @@ grib_runtime_options global_options={
 
 static grib_handle* grib_handle_new_from_file_x(grib_context* c,FILE* f,int mode,int headers_only,int *err)
 {
+    if (mode==MODE_GRIB)
+        return grib_new_from_file(c,f,headers_only,err);
+
+    if (mode==MODE_BUFR)
+        return bufr_new_from_file(c,f,err);
+
     if (mode==MODE_ANY)
         return any_new_from_file(c,f,err);
 
     if (mode==MODE_GTS)
         return gts_new_from_file(c,f,err);
 
-    if (mode==MODE_BUFR)
-        return bufr_new_from_file(c,f,err);
-
     if (mode==MODE_METAR)
         return metar_new_from_file(c,f,err);
 
     if (mode==MODE_TAF)
         return taf_new_from_file(c,f,err);
 
-    if (mode==MODE_GRIB)
-        return grib_new_from_file(c,f,headers_only,err);
-
     Assert(!"grib_handle_new_from_file_x: unknown mode");
     return NULL;
 }
@@ -674,7 +673,7 @@ static void grib_tools_set_print_keys(grib_runtime_options* options, grib_handle
     }
 
     if (ns) {
-        kiter=grib_keys_iterator_new(h,0,(char*)ns);
+        kiter=grib_keys_iterator_new(h,0,ns);
         if (!kiter) {
             fprintf(dump_file,"ERROR: Unable to create keys iterator\n");
             exit(1);
